@@ -3,14 +3,13 @@
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "2.2.0"
+__version__ = "2.2.3"
 __all__ = ['Model', 'Wizard']
 from types import NoneType
 import threading
 import datetime
 from decimal import Decimal
 import proteus.config
-from proteus.pyson import PYSONDecoder
 
 _MODELS = threading.local()
 
@@ -91,6 +90,14 @@ class CharDescriptor(FieldDescriptor):
         super(CharDescriptor, self).__set__(instance, value or '')
 
 
+class BinaryDescriptor(FieldDescriptor):
+    default = None
+
+    def __set__(self, instance, value):
+        assert isinstance(value, (basestring, buffer)) or value in (None, False)
+        super(BinaryDescriptor, self).__set__(instance, value or '')
+
+
 class IntegerDescriptor(FieldDescriptor):
     default = 0
 
@@ -163,7 +170,8 @@ class Many2OneDescriptor(FieldDescriptor):
             value = relation(value)
         elif not value:
             value = None
-        instance._values[self.name] = value
+        if self.name in instance._values:
+            instance._values[self.name] = value
         return value
 
     def __set__(self, instance, value):
@@ -295,7 +303,7 @@ class MetaModelFactory(object):
         'char': CharDescriptor,
         'text': CharDescriptor,
         'sha': CharDescriptor,
-        'binary': CharDescriptor,
+        'binary': BinaryDescriptor,
         'selection': CharDescriptor, # TODO implement its own descriptor
         'integer': IntegerDescriptor,
         'biginteger': IntegerDescriptor,
@@ -478,7 +486,7 @@ class Model(object):
     _config = None
     _fields = None
 
-    def __init__(self, id=None, **kwargs):
+    def __init__(self, id=None, _default=True, **kwargs):
         super(Model, self).__init__()
         if id:
             assert not kwargs
@@ -490,7 +498,7 @@ class Model(object):
         self._parent = None # store the parent record
         self._parent_field_name = '' # store the field name in parent record
         self._parent_name = '' # store the field name to parent record
-        if self.id < 0:
+        if self.id < 0 and _default:
             self._default_get()
 
         for field_name, value in kwargs.iteritems():
@@ -572,7 +580,7 @@ class Model(object):
         'Save the record'
         context = self._config.context
         if self.id < 0:
-            values = self._get_values(fields=self._changed)
+            values = self._get_values()
             self.__id = self._proxy.create(values, context)
         else:
             if not self._changed:
@@ -683,11 +691,8 @@ class Model(object):
         return res
 
     def _on_change_set(self, field, value):
-        if self._fields[field]['type'] in ('one2many', 'many2many'):
-            if isinstance(value, (list, tuple)):
-                self._values[field] = value
-                self._changed.add(field)
-                return
+        if (self._fields[field]['type'] in ('one2many', 'many2many')
+                and not isinstance(value, (list, tuple))):
             to_remove = []
             if value and value.get('remove'):
                 for record_id in value['remove']:
@@ -701,22 +706,32 @@ class Model(object):
                 for vals in value.get('add', []):
                     relation = Model.get(self._fields[field]['relation'],
                             self._config)
+                    record = relation(_default=False)
+                    for i, j in vals.iteritems():
+                        if '.' in i:
+                            continue
+                        record._values[i] = j
+                        record._changed.add(i)
                     # append without signal
-                    list.append(getattr(self, field), relation(**vals))
+                    list.append(getattr(self, field), record)
                 for vals in value.get('update', []):
                     if 'id' not in vals:
                         continue
                     for record in getattr(self, field):
                         if record.id == vals['id']:
                             for i, j in vals.iteritems():
+                                if '.' in i:
+                                    continue
                                 record._values[i] = j
                                 record._changed.add(i)
         else:
             self._values[field] = value
-            self._changed.add(field)
+        self._changed.add(field)
 
     def _on_change(self, name):
         'Call on_change for field'
+        # Import locally to not break installation
+        from proteus.pyson import PYSONDecoder
         definition = self._fields[name]
         if definition.get('on_change'):
             if isinstance(definition['on_change'], basestring):
@@ -735,8 +750,6 @@ class Model(object):
                 self._on_change_set(field, value)
             for field, value in later.iteritems():
                 self._on_change_set(field, value)
-            if self._parent:
-                self._parent._changed.add(self._parent_field_name)
         for field, definition in self._fields.iteritems():
             if not definition.get('on_change_with'):
                 continue
@@ -749,6 +762,9 @@ class Model(object):
             res = getattr(self._proxy, 'on_change_with_%s' % field)(args,
                     context)
             self._on_change_set(field, res)
+        if self._parent:
+            self._parent._changed.add(self._parent_field_name)
+            self._parent._on_change(self._parent_field_name)
 
 
 class Wizard(object):
